function buildAdvancedCompGraph(obj)

    fprintf("Building Advanced Computational Graph.\n");

    obj.ndFlagFiltSurf = comg.Node("Filtering cylindrical surfaces", false);
    obj.ndFiltSurfAngMean = comg.Node("Filtering cylindrical mean angle", -20);
    obj.ndFiltSurfAngRang = comg.Node("Filtering cylindrical angle range", 10);
    obj.ndFiltSurfThre = comg.Node("Filtering cylindrical threshold", 0);

    ndShowSurf = obj.ndShowSurf;
    obj.ndShowSurf = comg.Node.derived("Filtered Surface", ...
        @drvFiltSurf, ...
        ndShowSurf, obj.ndFlagFiltSurf, obj.ndFiltSurfAngMean, obj.ndFiltSurfAngRang, obj.ndFiltSurfThre);

end

function filt_surf = drvFiltSurf(surf, flag, mean_ang, range, peak_dist)
    if flag
        filt_surf = filtSurf(surf, mean_ang, range, peak_dist);
    else
        filt_surf = surf;
    end
end

function filt_surf = filtSurf(surf, mean_ang, range, peak_dist)
    
    [sy, sx] = cencoord(size(surf), 1./size(surf));
    r2 = (sy.*size(surf,1)).^2 + (sx.*size(surf,2)).^2;
    sa = atan2d(sy, sx);
    sa = angle(exp(1j*2*pi/360*(sa-mean_ang)))/pi*180;
    msk = (abs(sa) < range) | (r2 < 3^2);
    msk = msk | msk(end:-1:1, end:-1:1);

    filt_surf = surf;

    filt_surfbg = movmedian(movmedian(filt_surf, 60, 1), 60, 2);
    filt_surf = filt_surf - filt_surfbg;

    weight = movmax(movmax(msk, 11, 1), 11, 2);
    filt_surf = real(cifft2(cfft2(filt_surf).*weight));

    glob_med = median(filt_surf(:));
    filt_surf = min(glob_med, filt_surf);

    if peak_dist > 0
        peak_map = zeros(size(filt_surf));
        for h = 1:size(filt_surf)
            [~,locs] = findpeaks(-filt_surf(h,:), 'MinPeakDistance', peak_dist);
            id = locs + (-5:5)';
            id = min(size(filt_surf, 2), max(1, id(:)));
            peak_map(h, id) = filt_surf(h, id);
        end
        filt_surf = peak_map;
    end
end


