function registerTomoViewAxes(obj, ax)
    initTomoViewAxes(ax);

    click_callback = @(src, evt)(obj.tomoViewLayerClickedCallback(src, evt));
    ndShowTomoLay = comg.Node.output("ShowTomoLay", ...
            @(tomovol, layind)(showTomoLay(ax, click_callback, tomovol, layind)), ...
            obj.ndTomoVol, obj.ndTomoLayInd...
        ).activate();

    comg.Node.output("ShowCreatingFila", ...
            @(pts)(showCreatingFila(ax, pts)), ...
            obj.ndCrtFilaPts ...
        ).activate().after(ndShowTomoLay);

    comg.Node.output("ShowSelectingFila", ...
            @(pts)(showSelectingFila(ax, pts)), ...
            obj.ndSelFilaPts ...
        ).activate().after(ndShowTomoLay);
    
    comg.Node.output("ShowAllFila", ...
            @(pts)(showAllFila(ax, pts)), ...
            obj.ndAllFilaPts ...
        ).activate().after(ndShowTomoLay);

    ndShowTuneFila = comg.Node.output("ShowTuningFila", ...
            @(pts)(showTuningFila(ax, pts)), ...
            obj.ndExtrFilaPts ...
        ).activate().after(ndShowTomoLay);

    comg.Node.output("ShowTunedFila", ...
            @(pts, focpt)(showTunedFila(ax, pts, focpt)), ...
            obj.ndTFNewFilaPts, obj.ndTFFocPt ...
        ).activate().after(ndShowTuneFila);

    comg.Node.output("Show Current Point", ...
            @(currpt, flag_show)(showCurrPoint(ax, currpt, flag_show)), ...
           obj.ndTFCurrPtEul, obj.ndStatTFShowCurrPt ...
        ).activate().after(ndShowTuneFila);

end

function initTomoViewAxes(tomoview_ax)
    xlabel(tomoview_ax, "X-aixs"); % (fastest axis in tomo data)
    ylabel(tomoview_ax, "Y-aixs");
    zlabel(tomoview_ax, "Z-aixs");
    colormap(tomoview_ax, 'gray');
    tomoview_ax.View = [-80, 70];
    hold(tomoview_ax, "on");
    axis(tomoview_ax, "equal");
    title(tomoview_ax, "Tomogram");
end

function showTomoLay(tomoview_ax, click_callback, tomovol, layind)
    persistent handle_tomolay;

    lowlim = ceil(median(size(tomovol)) * 0.7);
    showsz = max(lowlim, size(tomovol));
    showbias = cenpos(size(tomovol)) - cenpos(showsz);
    
    xlim(tomoview_ax, 0.5+showbias(1)+[0, showsz(1)]);
    ylim(tomoview_ax, 0.5+showbias(2)+[0, showsz(2)]);
    zlim(tomoview_ax, 0.5+showbias(3)+[0, showsz(3)]);

    xImage = 0.5+[0, size(tomovol,1); 0, size(tomovol,1)];   % The x data for the image corners
    yImage = 0.5+[0, 0; size(tomovol,2), size(tomovol,2)];   % The y data for the image corners
    zImage = layind * [1, 1; 1, 1];             % The z data for the image corners

    img = tomovol(:,:,layind)';

    delete(handle_tomolay);
    handle_tomolay = surf(tomoview_ax, xImage,yImage,zImage,...    % Plot the surface
         'CData',img * 255, ...
         'FaceColor', 'texturemap',...
         'EdgeColor', 'none', ...
         'ButtonDownFcn', click_callback);
    alpha(handle_tomolay, 0.9);
end

function [handle_cv, handle_pts] = drawFila(ax, pts, color)
    handle_cv = [];
    handle_pts = [];
    if ~isempty(pts)
        if size(pts, 2) >= 1
            handle_pts = plot3( ...
                ax, ...
                pts(1,:), pts(2,:), pts(3,:), ...
                'Marker', 'o', 'LineStyle', 'none',...
                'Color', color, ...
                'LineWidth', 2, 'PickableParts', 'none');
        end

        if size(pts, 2) >= 2
            smoothed_pts = helix.filamentSmoothing(pts);
            
            handle_cv = plot3( ...
                ax, ...
                smoothed_pts(1,:), smoothed_pts(2,:), smoothed_pts(3,:), ...
                'Color', color, ...
                'LineWidth', 2, 'PickableParts', 'none');
        end
    end
end

function showCreatingFila(tomoview_ax, pts)
    persistent handle_filacv;
    persistent handle_filapts;

    color = [.8, .4, .4];

    delete(handle_filacv);
    delete(handle_filapts);
    [handle_filacv, handle_filapts] = drawFila(tomoview_ax, pts, color);
end

function showSelectingFila(tomoview_ax, pts)
    persistent handle_filacv;
    persistent handle_filapts;

    color = [.4, .4, .8];
    
    delete(handle_filacv);
    delete(handle_filapts);
    [handle_filacv, handle_filapts] = drawFila(tomoview_ax, pts, color);
end

function showAllFila(tomoview_ax, pts)
    persistent handle_filacv;
    persistent handle_filapts;

    color = [.4, .8, .4];
    
    for ii = 1:numel(handle_filacv)
        delete(handle_filacv{ii});
    end
    for ii = 1:numel(handle_filapts)
        delete(handle_filapts{ii});
    end
    if ~isempty(pts)
        handle_filacv = cell(1, numel(pts));
        handle_filapts = cell(1, numel(pts));
        for ii = 1:numel(pts)
            [handle_filacv{ii}, handle_filapts{ii}] = drawFila(tomoview_ax, pts{ii}, color);
        end
    end
end

function showTuningFila(tomoview_ax, pts)
    persistent handle_filacv;
    persistent handle_filapts;

    color = [.8, .4, .4];

    delete(handle_filacv);
    delete(handle_filapts);
    [handle_filacv, handle_filapts] = drawFila(tomoview_ax, pts, color);
end

function showTunedFila(tomoview_ax, pts, focpt)
    persistent handle_filacv;
    persistent handle_filapts;
    persistent handle_focpt;

    tuned_color = [.6, .6, .4];
    foc_color = [.4, .4, .8];
    
    delete(handle_filacv);
    delete(handle_filapts);
    delete(handle_focpt);
    [handle_filacv, handle_filapts] = drawFila(tomoview_ax, pts, tuned_color);
    [~, handle_focpt] = drawFila(tomoview_ax, focpt, foc_color);
end

function showCurrPoint(tomoview_ax, currpt, flag_show)
    persistent handle_pt;

    curr_color = [.4, .8, .4];
    
    delete(handle_pt);
    if flag_show && ~isempty(currpt)
        handle_pt = plot3( ...
            tomoview_ax, ...
            currpt(1), currpt(2), currpt(3), ...
            'Marker', 'x', 'LineStyle', 'none',...
            'Color', curr_color, ...
            'LineWidth', 2, 'PickableParts', 'none');
    end
end




