function [trace_pos, trace_dir, trace_score, rand_scores] = filamentTracing3Dfast(vol, init_pts, extr_rad, temp_rad,...
    seg_len, sear_step, sear_numb, sear_range, varargin)

    persistent p;
    if isempty(p)
        p = inputParser();
        p.addParameter('est_rand_scores', false);
        p.addParameter('num_rand_scores', 100);
        p.addParameter('score_thres', []);
        p.addParameter('border_dist', []);
    end
    p.parse(varargin{:});

    % extracting selected filament segment to construct template.
    [~, fila_axis_func, fila_axis_len, ~] = TomoFilamentViewer.filamentSmoothing(init_pts);
    [extr_fila,~,~,~,~,~,~,~] = filamentExtraction(vol, fila_axis_func, 1, fila_axis_len, extr_rad);
    [~, ~, templ] = ring_average(sum(extr_fila,3));
    templ = templ - mean(templ,'all');
    templ_cylin = cenrsz(repmat(templ, [1,1,seg_len]),[temp_rad*2+1, temp_rad*2+1, seg_len]);
    loose_templ_sz = ceil(sqrt(sum(size(templ_cylin).^2)));
    loose_templ_sz = loose_templ_sz + 1 - mod(loose_templ_sz,2);
%     templ_cylin_model = VoxelModel3D(cenrsz(templ_cylin, loose_templ_sz*[1,1,1]));
    templ_cylin_val = cenrsz(templ_cylin, loose_templ_sz*[1,1,1]);
    
    volsz = size(templ_cylin_val);
    [vx, vy, vz] = cencoord(volsz);
    vcd = [vx(:),vy(:),vz(:)];
    vxb = cencoord(volsz(1));
    vyb = cencoord(volsz(2));
    vzb = cencoord(volsz(3));

    % estimate_rand_score
    rand_scores = [];
    if p.Results.est_rand_scores
        Nest = p.Results.num_rand_scores;
        rand_scores = zeros(1, Nest);
        for i = 1:Nest
            rand_poses = randn(3,i).*(size(vol)' - loose_templ_sz) + round(loose_templ_sz);
            rand_extr_vols = get_subvols(vol, rand_poses, loose_templ_sz,'ignore_sub_pixel',true);

%             rot_templ = templ_cylin_model.rotated_val(randq(1));
            
            vcd_rt = randq(1).rotateframe(vcd);
            rot_templ = reshape(interp3(vyb, vxb, vzb, templ_cylin_val, vcd_rt(:,2), vcd_rt(:,1), vcd_rt(:,3), 'linear', 0), volsz);

            rand_scores(i) = sum(rot_templ .* rand_extr_vols, 'all') ./ norm(rot_templ(:)) ./ norm(rand_extr_vols(:));
        end
    end


    % start filament tracing
    trace_pos = zeros(3, sear_numb);
    trace_dir = zeros(3, sear_numb);
    trace_score = zeros(1, sear_numb);
    
    current_pos = fnval(fila_axis_func, fila_axis_len);
    current_dir = normalize(diff(fnval(fila_axis_func, fila_axis_len+[-0.5e-4,0.5e-4]),[],2),'norm');
    
    flag_exit = false;
    for isearch = 1:sear_numb
        fprintf("Searching the %d-th (of %d) points.\n", isearch, sear_numb);
    
        cand_pos = current_pos + current_dir * sear_step;

        comp_vol = get_subvols(vol, current_pos, loose_templ_sz+sear_range,'ignore_sub_pixel',true);
        
%         rot_templ = templ_cylin_model.rotated_val(quat_mapztov(current_dir));
        vcd_rt = quat_mapztov(current_dir).rotateframe(vcd);
        rot_templ = reshape(interp3(vyb, vxb, vzb, templ_cylin_val, vcd_rt(:,2), vcd_rt(:,1), vcd_rt(:,3), 'linear', 0), volsz);

        [coef, coef_cent] = tempmatch(rot_templ, comp_vol);
        [max_coef, ind] = max(coef, [], 'all');
        trace_score(isearch) = max_coef;

        [mcx,mcy,mcz] = ind2sub(size(coef), ind);
        mc_pos = [mcx, mcy, mcz] - coef_cent;

        new_pos = cand_pos + mc_pos';
        new_dir = normalize(new_pos - current_pos, 'norm');

        trace_pos(:,isearch) = new_pos;
        trace_dir(:,isearch) = new_dir;

        current_pos = new_pos;
        current_dir = new_dir;

        if ~isempty(p.Results.score_thres)
            if max_coef < p.Results.score_thres
                flag_exit = true;
            end
        end
        if ~isempty(p.Results.border_dist)
            bd = p.Results.border_dist(:);
            if any(new_pos < bd) || any(new_pos > size(vol)'-bd)
                flag_exit = true;
            end
        end

        if flag_exit
            trace_pos = trace_pos(:, 1:isearch - 1);
            trace_dir = trace_dir(:, 1:isearch - 1);
            trace_score = trace_score(:, 1:isearch - 1);
            break;
        end

    end
    
    

end










