function symm_vol = volHelicalSymmetrize(vol, msn, rise, twist, order)
%   msn: minminal start number, or the fold number of rotation symmetize
%   rise: in pixel
%   twist: in degree
%   order: order of implementing helical symmetry

    volsz = size(vol);
    [vx, vy, vz] = cencoord(volsz);
    vcd = [vx(:),vy(:),vz(:)];
    vxb = cencoord(volsz(1));
    vyb = cencoord(volsz(2));
    vzb = cencoord(volsz(3));

    if order >= 1
        v = vol;
        vn = vol;
        for logord = 0:(floor(log2(order))-1)
            ord = 2^logord;

            vcd_rt = quaternion([0,0,twist*ord],'rotvecd').rotateframe(vcd) - [0,0,rise*ord];
            vol_rt = reshape(interp3(vyb, vxb, vzb, v, vcd_rt(:,2), vcd_rt(:,1), vcd_rt(:,3), 'linear', 0), volsz);
            v = (v + vol_rt)/2;

            vcd_rt = quaternion([0,0,-twist*ord],'rotvecd').rotateframe(vcd) - [0,0,-rise*ord];
            vol_rt = reshape(interp3(vyb, vxb, vzb, vn, vcd_rt(:,2), vcd_rt(:,1), vcd_rt(:,3), 'linear', 0), volsz);
            vn = (vn + vol_rt)/2;
        end
        
        symm_vol = (v + vn) / 2;
    end

    if msn > 1
        v = symm_vol;
        rot_ang = 360/msn*(1:msn-1);
        for ir = 1:msn
            vcd_rt = quaternion([0,0,ir*rot_ang],'rotvecd').rotateframe(vcd);
            vol_rt = reshape(interp3(vyb, vxb, vzb, v, vcd_rt(:,2), vcd_rt(:,1), vcd_rt(:,3), 'linear', 0), volsz);
            symm_vol = symm_vol + vol_rt;
        end
        symm_vol = symm_vol ./ msn;
    end

end




