classdef helis_exported < matlab.apps.AppBase

    % Properties that correspond to app components
    properties (Access = public)
        UIFigure                  matlab.ui.Figure
        GridLayout2               matlab.ui.container.GridLayout
        GridLayout3               matlab.ui.container.GridLayout
        GridLayout5               matlab.ui.container.GridLayout
        NotesTextArea             matlab.ui.control.TextArea
        NotesTextAreaLabel        matlab.ui.control.Label
        GridLayout4               matlab.ui.container.GridLayout
        LoadLabel                 matlab.ui.control.Label
        FilaButton                matlab.ui.control.Button
        TomoButton                matlab.ui.control.Button
        thresEditField            matlab.ui.control.NumericEditField
        thresEditFieldLabel       matlab.ui.control.Label
        AbortButton               matlab.ui.control.Button
        DeleteButton              matlab.ui.control.Button
        TraceButton               matlab.ui.control.Button
        CenteringButton           matlab.ui.control.Button
        FinishTuningButton        matlab.ui.control.Button
        SFSTAButton               matlab.ui.control.Button
        CancelButton              matlab.ui.control.Button
        UpdateButton              matlab.ui.control.Button
        OptVsButton               matlab.ui.control.Button
        SetVbButton               matlab.ui.control.Button
        SetVaButton               matlab.ui.control.Button
        RmvptButton               matlab.ui.control.Button
        MovptButton               matlab.ui.control.Button
        AddptButton               matlab.ui.control.Button
        PrevptButton              matlab.ui.control.Button
        NextptButton              matlab.ui.control.Button
        TuneFilamentButton        matlab.ui.control.Button
        FilamentsListBox          matlab.ui.control.ListBox
        FilamentsListBoxLabel     matlab.ui.control.Label
        SaveButton                matlab.ui.control.Button
        ShowButton                matlab.ui.control.Button
        CloseButton               matlab.ui.control.Button
        UndoButton                matlab.ui.control.Button
        AcceptButton              matlab.ui.control.Button
        AddFilamentButton         matlab.ui.control.Button
        TabGroup                  matlab.ui.container.TabGroup
        TomogramTab               matlab.ui.container.Tab
        GridLayout6               matlab.ui.container.GridLayout
        FilpContrastButton        matlab.ui.control.StateButton
        LayeridxSpinner           matlab.ui.control.Spinner
        LayeridxSpinnerLabel      matlab.ui.control.Label
        NLabel                    matlab.ui.control.Label
        UIAxes                    matlab.ui.control.UIAxes
        FilamentTab               matlab.ui.container.Tab
        GridLayout7               matlab.ui.container.GridLayout
        saveSurfButton            matlab.ui.control.Button
        findpkEditField           matlab.ui.control.NumericEditField
        findpkEditFieldLabel      matlab.ui.control.Label
        rangEditField             matlab.ui.control.NumericEditField
        rangEditFieldLabel        matlab.ui.control.Label
        meanEditField             matlab.ui.control.NumericEditField
        meanEditFieldLabel        matlab.ui.control.Label
        surffilterButton          matlab.ui.control.StateButton
        avgsurfButton             matlab.ui.control.StateButton
        TdegEditField             matlab.ui.control.NumericEditField
        TdegEditFieldLabel        matlab.ui.control.Label
        RpxlEditField             matlab.ui.control.NumericEditField
        RpxlEditFieldLabel        matlab.ui.control.Label
        MinstatnumEditField       matlab.ui.control.NumericEditField
        MinstatnumEditFieldLabel  matlab.ui.control.Label
        aEditFieldLabel_3         matlab.ui.control.Label
        piEditField_2             matlab.ui.control.NumericEditField
        piLabel                   matlab.ui.control.Label
        mathbfbEditField          matlab.ui.control.NumericEditField
        mathbfbEditFieldLabel     matlab.ui.control.Label
        aEditFieldLabel_2         matlab.ui.control.Label
        piEditField               matlab.ui.control.NumericEditField
        Label                     matlab.ui.control.Label
        mathbfaEditField          matlab.ui.control.NumericEditField
        mathbfaLabel              matlab.ui.control.Label
        DeltaLSpinnerLabel        matlab.ui.control.Label
        DeltaLSpinner             matlab.ui.control.Spinner
        DeltanSpinner             matlab.ui.control.Spinner
        DeltanSpinnerLabel        matlab.ui.control.Label
        MaxorderSpinner           matlab.ui.control.Spinner
        MaxorderSpinnerLabel      matlab.ui.control.Label
        SegstepSpinner            matlab.ui.control.Spinner
        SegstepSpinnerLabel       matlab.ui.control.Label
        SegsizeSpinner            matlab.ui.control.Spinner
        SegsizeSpinnerLabel       matlab.ui.control.Label
        log10GammaSpectrumLabel   matlab.ui.control.Label
        rmvbgButton               matlab.ui.control.StateButton
        GammaSpinner              matlab.ui.control.Spinner
        GammaSpinnerLabel         matlab.ui.control.Label
        OuterradSpinner           matlab.ui.control.Spinner
        OuterradSpinnerLabel      matlab.ui.control.Label
        InnerradSpinner           matlab.ui.control.Spinner
        InnerradSpinnerLabel      matlab.ui.control.Label
        AvgnumSpinner             matlab.ui.control.Spinner
        AvgnumSpinnerLabel        matlab.ui.control.Label
        NLabel_2                  matlab.ui.control.Label
        SectionposSpinner         matlab.ui.control.Spinner
        SectionposSpinnerLabel    matlab.ui.control.Label
        UIAxes4                   matlab.ui.control.UIAxes
        UIAxes3                   matlab.ui.control.UIAxes
        UIAxes2                   matlab.ui.control.UIAxes
        ContextMenu               matlab.ui.container.ContextMenu
        AutoEstimateMenu          matlab.ui.container.Menu
    end

    % Callbacks that handle component events
    methods (Access = private)

        % Code that executes after component creation
        function startupFcn(app)
            
            fv = helix.TomoFilamentUIManager();

            if exist('tomo_fn', 'var') && exist(tomo_fn, 'file')
                [vol, ~] = readmrc(tomo_fn);
                vol = single(vol);
                if exist('flag_rotx', 'var') && flag_rotx
                    vol = permute(vol, [1,3,2]);
                    vol = vol(:,:,end:-1:1);
                end
%                 uplim = prctile(vol(:), 99);
%                 lowlim = prctile(vol(:), 1);
%                 vol = max(lowlim, min(uplim, vol));
                fv.loadTomogramData(vol);
            end

            fd = helix.FilamentDataHandle();
            if exist('fila_fn', 'var') && exist(fila_fn, 'file')
                fd.loadFromFile(fila_fn);
            end

            eng = helix.FilamentEditingEngine();
            eng.registerFilamentDataHandle(fd);

            fv.registerFilamentEditingEngine(eng);

            fv.registerMainFigure(app.UIFigure);

            fv.registerTomoViewAxes(app.UIAxes);
            fv.registerCylinAxes(app.UIAxes2);
            fv.registerCylinFTAxes(app.UIAxes4);
            fv.registerFilaSecAxes(app.UIAxes3);

            fv.registerLoadTomoButton(app.TomoButton);
            fv.registerLoadFilaButton(app.FilaButton);

            fv.registerFlipContButton(app.FilpContrastButton);
            fv.registerAddFilaButton(app.AddFilamentButton);
            fv.registerAccFilaButton(app.AcceptButton);
            fv.registerUndFilaButton(app.UndoButton);
            fv.registerClsFilaButton(app.CloseButton);
            fv.registerDelFilaButton(app.DeleteButton);
            fv.registerShowAllFilaButton(app.ShowButton);
            fv.registerSaveFilaButton(app.SaveButton);
            fv.registerTuneFilaButton(app.TuneFilamentButton);
            fv.registerTFNextptButton(app.NextptButton);
            fv.registerTFPrevptButton(app.PrevptButton);
            fv.registerTFAddptButton(app.AddptButton);
            fv.registerTFMovptButton(app.MovptButton);
            fv.registerTFRmvptButton(app.RmvptButton);
            fv.registerTFConfButton(app.UpdateButton);
            fv.registerTFAbortButton(app.AbortButton);
            fv.registerTFCancButton(app.CancelButton);
            fv.registerTFDoneButton(app.FinishTuningButton);
            fv.registerTFSetvaButton(app.SetVaButton);
            fv.registerTFSetvbButton(app.SetVbButton);
            fv.registerTFOptRpvButton(app.OptVsButton);
            fv.registerTFAvgSurfButton(app.avgsurfButton);
            fv.registerTFReconButton(app.SFSTAButton);
            fv.registerTFSpecRmvbgButton(app.rmvbgButton);
            fv.registerTFFilaAutoCenButton(app.CenteringButton);
            fv.registerTFFilaTraceButton(app.TraceButton);

            fv.registerFilaListBox(app.FilamentsListBox);

            fv.registerNlaySpin(app.LayeridxSpinner);
            fv.registerNlayLabel(app.NLabel);
            fv.registerSecLenSpin(app.SectionposSpinner);
            fv.registerSecLenLabel(app.NLabel_2);

            fv.registerAvgNumSpin(app.AvgnumSpinner);
            fv.registerInnRadSpin(app.InnerradSpinner);
            fv.registerOutRadSpin(app.OuterradSpinner);

            fv.registerCftGammSpin(app.GammaSpinner);
            fv.registerSegSizeSpin(app.SegsizeSpinner);
            fv.registerSegStepSpin(app.SegstepSpinner);
            fv.registerMaxOrdSpin(app.MaxorderSpinner);
            fv.registerTFOptLenSpin(app.DeltaLSpinner);
            fv.registerTFOptAngSpin(app.DeltanSpinner);

            fv.registerTFTraceThresEdit(app.thresEditField);

            fv.registerVaaEdit(app.mathbfaEditField);
            fv.registerValEdit(app.piEditField);
            fv.registerVbaEdit(app.mathbfbEditField);
            fv.registerVblEdit(app.piEditField_2);

            fv.registerHpcnEdit(app.MinstatnumEditField);
            fv.registerHprsEdit(app.RpxlEditField);
            fv.registerHptwEdit(app.TdegEditField);

            fv.registerSaveSurfButton(app.saveSurfButton);

            %%% Advanced 
%             fv.registerTFSurfFilterButton(app.surffilterButton);
%             fv.registerTFSurfFiltAngMeanSpin(app.meanEditField);
%             fv.registerTFSurfFiltAngRangeSpin(app.rangEditField);
%             fv.registerTFSurfFiltThreSpin(app.findpkEditField);

        end
    end

    % Component initialization
    methods (Access = private)

        % Create UIFigure and components
        function createComponents(app)

            % Create UIFigure and hide until all components are created
            app.UIFigure = uifigure('Visible', 'off');
            app.UIFigure.Position = [100 100 1411 702];
            app.UIFigure.Name = 'MATLAB App';

            % Create GridLayout2
            app.GridLayout2 = uigridlayout(app.UIFigure);
            app.GridLayout2.ColumnWidth = {'1x', '2.6x'};
            app.GridLayout2.RowHeight = {'1x'};

            % Create TabGroup
            app.TabGroup = uitabgroup(app.GridLayout2);
            app.TabGroup.Layout.Row = 1;
            app.TabGroup.Layout.Column = 2;

            % Create TomogramTab
            app.TomogramTab = uitab(app.TabGroup);
            app.TomogramTab.Title = 'Tomogram';

            % Create GridLayout6
            app.GridLayout6 = uigridlayout(app.TomogramTab);
            app.GridLayout6.ColumnWidth = {'28.31x', 54, '2.15x', 63};
            app.GridLayout6.RowHeight = {'1x', '1x', '1x', '1x', '12x'};
            app.GridLayout6.RowSpacing = 0.5;
            app.GridLayout6.Padding = [10 0.5 10 0.5];

            % Create UIAxes
            app.UIAxes = uiaxes(app.GridLayout6);
            title(app.UIAxes, 'Title')
            xlabel(app.UIAxes, 'X')
            ylabel(app.UIAxes, 'Y')
            zlabel(app.UIAxes, 'Z')
            app.UIAxes.Layout.Row = [1 5];
            app.UIAxes.Layout.Column = 1;

            % Create NLabel
            app.NLabel = uilabel(app.GridLayout6);
            app.NLabel.Layout.Row = 2;
            app.NLabel.Layout.Column = 4;
            app.NLabel.Text = '/N';

            % Create LayeridxSpinnerLabel
            app.LayeridxSpinnerLabel = uilabel(app.GridLayout6);
            app.LayeridxSpinnerLabel.Layout.Row = 2;
            app.LayeridxSpinnerLabel.Layout.Column = 2;
            app.LayeridxSpinnerLabel.Text = 'Layer idx';

            % Create LayeridxSpinner
            app.LayeridxSpinner = uispinner(app.GridLayout6);
            app.LayeridxSpinner.Limits = [1 Inf];
            app.LayeridxSpinner.RoundFractionalValues = 'on';
            app.LayeridxSpinner.Layout.Row = 2;
            app.LayeridxSpinner.Layout.Column = 3;
            app.LayeridxSpinner.Value = 1;

            % Create FilpContrastButton
            app.FilpContrastButton = uibutton(app.GridLayout6, 'state');
            app.FilpContrastButton.Text = 'Filp Contrast';
            app.FilpContrastButton.Layout.Row = 3;
            app.FilpContrastButton.Layout.Column = [2 3];

            % Create FilamentTab
            app.FilamentTab = uitab(app.TabGroup);
            app.FilamentTab.Title = 'Filament';

            % Create GridLayout7
            app.GridLayout7 = uigridlayout(app.FilamentTab);
            app.GridLayout7.ColumnWidth = {'6x', '4x', '3x', '1x', '1x', '1x', '1x', '1x', '1x'};
            app.GridLayout7.RowHeight = {'1x', '1x', '1x', '1x', '1x', '1x', '1x', '1x', '1x', '1x', '1x', '1x', '1x', '1x', '1x'};

            % Create UIAxes2
            app.UIAxes2 = uiaxes(app.GridLayout7);
            title(app.UIAxes2, 'Title')
            xlabel(app.UIAxes2, 'X')
            ylabel(app.UIAxes2, 'Y')
            zlabel(app.UIAxes2, 'Z')
            app.UIAxes2.Layout.Row = [1 15];
            app.UIAxes2.Layout.Column = 1;

            % Create UIAxes3
            app.UIAxes3 = uiaxes(app.GridLayout7);
            title(app.UIAxes3, 'Title')
            xlabel(app.UIAxes3, 'X')
            ylabel(app.UIAxes3, 'Y')
            zlabel(app.UIAxes3, 'Z')
            app.UIAxes3.Layout.Row = [1 7];
            app.UIAxes3.Layout.Column = [2 3];

            % Create UIAxes4
            app.UIAxes4 = uiaxes(app.GridLayout7);
            title(app.UIAxes4, 'Title')
            xlabel(app.UIAxes4, 'X')
            ylabel(app.UIAxes4, 'Y')
            zlabel(app.UIAxes4, 'Z')
            app.UIAxes4.Layout.Row = [8 15];
            app.UIAxes4.Layout.Column = [2 3];

            % Create SectionposSpinnerLabel
            app.SectionposSpinnerLabel = uilabel(app.GridLayout7);
            app.SectionposSpinnerLabel.Layout.Row = 1;
            app.SectionposSpinnerLabel.Layout.Column = [4 5];
            app.SectionposSpinnerLabel.Text = 'Section pos';

            % Create SectionposSpinner
            app.SectionposSpinner = uispinner(app.GridLayout7);
            app.SectionposSpinner.Limits = [1 Inf];
            app.SectionposSpinner.RoundFractionalValues = 'on';
            app.SectionposSpinner.Layout.Row = 1;
            app.SectionposSpinner.Layout.Column = [6 7];
            app.SectionposSpinner.Value = 1;

            % Create NLabel_2
            app.NLabel_2 = uilabel(app.GridLayout7);
            app.NLabel_2.Layout.Row = 1;
            app.NLabel_2.Layout.Column = 8;
            app.NLabel_2.Text = '/N';

            % Create AvgnumSpinnerLabel
            app.AvgnumSpinnerLabel = uilabel(app.GridLayout7);
            app.AvgnumSpinnerLabel.Layout.Row = 2;
            app.AvgnumSpinnerLabel.Layout.Column = [4 5];
            app.AvgnumSpinnerLabel.Text = 'Avg num';

            % Create AvgnumSpinner
            app.AvgnumSpinner = uispinner(app.GridLayout7);
            app.AvgnumSpinner.Limits = [1 30];
            app.AvgnumSpinner.RoundFractionalValues = 'on';
            app.AvgnumSpinner.Layout.Row = 2;
            app.AvgnumSpinner.Layout.Column = [6 7];
            app.AvgnumSpinner.Value = 1;

            % Create InnerradSpinnerLabel
            app.InnerradSpinnerLabel = uilabel(app.GridLayout7);
            app.InnerradSpinnerLabel.Layout.Row = 3;
            app.InnerradSpinnerLabel.Layout.Column = [4 5];
            app.InnerradSpinnerLabel.Text = 'Inner rad';

            % Create InnerradSpinner
            app.InnerradSpinner = uispinner(app.GridLayout7);
            app.InnerradSpinner.Limits = [1 Inf];
            app.InnerradSpinner.RoundFractionalValues = 'on';
            app.InnerradSpinner.Layout.Row = 3;
            app.InnerradSpinner.Layout.Column = [6 7];
            app.InnerradSpinner.Value = 5;

            % Create OuterradSpinnerLabel
            app.OuterradSpinnerLabel = uilabel(app.GridLayout7);
            app.OuterradSpinnerLabel.Layout.Row = 4;
            app.OuterradSpinnerLabel.Layout.Column = [4 5];
            app.OuterradSpinnerLabel.Text = 'Outer rad';

            % Create OuterradSpinner
            app.OuterradSpinner = uispinner(app.GridLayout7);
            app.OuterradSpinner.Limits = [1 Inf];
            app.OuterradSpinner.RoundFractionalValues = 'on';
            app.OuterradSpinner.Layout.Row = 4;
            app.OuterradSpinner.Layout.Column = [6 7];
            app.OuterradSpinner.Value = 10;

            % Create GammaSpinnerLabel
            app.GammaSpinnerLabel = uilabel(app.GridLayout7);
            app.GammaSpinnerLabel.Layout.Row = 6;
            app.GammaSpinnerLabel.Layout.Column = [4 5];
            app.GammaSpinnerLabel.Text = 'Gamma';

            % Create GammaSpinner
            app.GammaSpinner = uispinner(app.GridLayout7);
            app.GammaSpinner.Step = 0.25;
            app.GammaSpinner.Layout.Row = 6;
            app.GammaSpinner.Layout.Column = [6 7];

            % Create rmvbgButton
            app.rmvbgButton = uibutton(app.GridLayout7, 'state');
            app.rmvbgButton.Text = 'rmv bg';
            app.rmvbgButton.Layout.Row = 5;
            app.rmvbgButton.Layout.Column = [6 7];

            % Create log10GammaSpectrumLabel
            app.log10GammaSpectrumLabel = uilabel(app.GridLayout7);
            app.log10GammaSpectrumLabel.Layout.Row = 7;
            app.log10GammaSpectrumLabel.Layout.Column = [4 8];
            app.log10GammaSpectrumLabel.Text = 'log (10^Gamma + Spectrum)';

            % Create SegsizeSpinnerLabel
            app.SegsizeSpinnerLabel = uilabel(app.GridLayout7);
            app.SegsizeSpinnerLabel.Layout.Row = 8;
            app.SegsizeSpinnerLabel.Layout.Column = [4 5];
            app.SegsizeSpinnerLabel.Text = 'Seg size';

            % Create SegsizeSpinner
            app.SegsizeSpinner = uispinner(app.GridLayout7);
            app.SegsizeSpinner.Limits = [30 Inf];
            app.SegsizeSpinner.RoundFractionalValues = 'on';
            app.SegsizeSpinner.Layout.Row = 8;
            app.SegsizeSpinner.Layout.Column = [6 7];
            app.SegsizeSpinner.Value = 100;

            % Create SegstepSpinnerLabel
            app.SegstepSpinnerLabel = uilabel(app.GridLayout7);
            app.SegstepSpinnerLabel.Layout.Row = 9;
            app.SegstepSpinnerLabel.Layout.Column = [4 5];
            app.SegstepSpinnerLabel.Text = 'Seg step';

            % Create SegstepSpinner
            app.SegstepSpinner = uispinner(app.GridLayout7);
            app.SegstepSpinner.Limits = [15 Inf];
            app.SegstepSpinner.RoundFractionalValues = 'on';
            app.SegstepSpinner.Layout.Row = 9;
            app.SegstepSpinner.Layout.Column = [6 7];
            app.SegstepSpinner.Value = 50;

            % Create MaxorderSpinnerLabel
            app.MaxorderSpinnerLabel = uilabel(app.GridLayout7);
            app.MaxorderSpinnerLabel.Layout.Row = 10;
            app.MaxorderSpinnerLabel.Layout.Column = [4 5];
            app.MaxorderSpinnerLabel.Text = 'Max order';

            % Create MaxorderSpinner
            app.MaxorderSpinner = uispinner(app.GridLayout7);
            app.MaxorderSpinner.Limits = [10 100];
            app.MaxorderSpinner.RoundFractionalValues = 'on';
            app.MaxorderSpinner.Layout.Row = 10;
            app.MaxorderSpinner.Layout.Column = [6 7];
            app.MaxorderSpinner.Value = 30;

            % Create DeltanSpinnerLabel
            app.DeltanSpinnerLabel = uilabel(app.GridLayout7);
            app.DeltanSpinnerLabel.Interpreter = 'latex';
            app.DeltanSpinnerLabel.Layout.Row = 11;
            app.DeltanSpinnerLabel.Layout.Column = 4;
            app.DeltanSpinnerLabel.Text = '$\Delta n$';

            % Create DeltanSpinner
            app.DeltanSpinner = uispinner(app.GridLayout7);
            app.DeltanSpinner.Limits = [0 10];
            app.DeltanSpinner.RoundFractionalValues = 'on';
            app.DeltanSpinner.Layout.Row = 11;
            app.DeltanSpinner.Layout.Column = [5 6];
            app.DeltanSpinner.Value = 2;

            % Create DeltaLSpinner
            app.DeltaLSpinner = uispinner(app.GridLayout7);
            app.DeltaLSpinner.Limits = [0 30];
            app.DeltaLSpinner.RoundFractionalValues = 'on';
            app.DeltaLSpinner.Layout.Row = 11;
            app.DeltaLSpinner.Layout.Column = [8 9];
            app.DeltaLSpinner.Value = 2;

            % Create DeltaLSpinnerLabel
            app.DeltaLSpinnerLabel = uilabel(app.GridLayout7);
            app.DeltaLSpinnerLabel.Interpreter = 'latex';
            app.DeltaLSpinnerLabel.Layout.Row = 11;
            app.DeltaLSpinnerLabel.Layout.Column = 7;
            app.DeltaLSpinnerLabel.Text = '$\Delta L$';

            % Create mathbfaLabel
            app.mathbfaLabel = uilabel(app.GridLayout7);
            app.mathbfaLabel.Interpreter = 'latex';
            app.mathbfaLabel.HorizontalAlignment = 'right';
            app.mathbfaLabel.Layout.Row = 12;
            app.mathbfaLabel.Layout.Column = 4;
            app.mathbfaLabel.Text = '$\mathbf{a}^*=($';

            % Create mathbfaEditField
            app.mathbfaEditField = uieditfield(app.GridLayout7, 'numeric');
            app.mathbfaEditField.RoundFractionalValues = 'on';
            app.mathbfaEditField.Editable = 'off';
            app.mathbfaEditField.Layout.Row = 12;
            app.mathbfaEditField.Layout.Column = 5;

            % Create Label
            app.Label = uilabel(app.GridLayout7);
            app.Label.Interpreter = 'latex';
            app.Label.HorizontalAlignment = 'right';
            app.Label.Layout.Row = 12;
            app.Label.Layout.Column = 6;
            app.Label.Text = '/2$\pi$,';

            % Create piEditField
            app.piEditField = uieditfield(app.GridLayout7, 'numeric');
            app.piEditField.Editable = 'off';
            app.piEditField.Layout.Row = 12;
            app.piEditField.Layout.Column = 7;

            % Create aEditFieldLabel_2
            app.aEditFieldLabel_2 = uilabel(app.GridLayout7);
            app.aEditFieldLabel_2.Interpreter = 'latex';
            app.aEditFieldLabel_2.Layout.Row = 12;
            app.aEditFieldLabel_2.Layout.Column = 8;
            app.aEditFieldLabel_2.Text = '$\text{pix}^{-1})$';

            % Create mathbfbEditFieldLabel
            app.mathbfbEditFieldLabel = uilabel(app.GridLayout7);
            app.mathbfbEditFieldLabel.Interpreter = 'latex';
            app.mathbfbEditFieldLabel.HorizontalAlignment = 'right';
            app.mathbfbEditFieldLabel.Layout.Row = 13;
            app.mathbfbEditFieldLabel.Layout.Column = 4;
            app.mathbfbEditFieldLabel.Text = '$\mathbf{b}^*=($';

            % Create mathbfbEditField
            app.mathbfbEditField = uieditfield(app.GridLayout7, 'numeric');
            app.mathbfbEditField.RoundFractionalValues = 'on';
            app.mathbfbEditField.Editable = 'off';
            app.mathbfbEditField.Layout.Row = 13;
            app.mathbfbEditField.Layout.Column = 5;

            % Create piLabel
            app.piLabel = uilabel(app.GridLayout7);
            app.piLabel.Interpreter = 'latex';
            app.piLabel.HorizontalAlignment = 'right';
            app.piLabel.Layout.Row = 13;
            app.piLabel.Layout.Column = 6;
            app.piLabel.Text = '/2$\pi$,';

            % Create piEditField_2
            app.piEditField_2 = uieditfield(app.GridLayout7, 'numeric');
            app.piEditField_2.Editable = 'off';
            app.piEditField_2.Layout.Row = 13;
            app.piEditField_2.Layout.Column = 7;

            % Create aEditFieldLabel_3
            app.aEditFieldLabel_3 = uilabel(app.GridLayout7);
            app.aEditFieldLabel_3.Interpreter = 'latex';
            app.aEditFieldLabel_3.Layout.Row = 13;
            app.aEditFieldLabel_3.Layout.Column = 8;
            app.aEditFieldLabel_3.Text = '$\text{pix}^{-1})$';

            % Create MinstatnumEditFieldLabel
            app.MinstatnumEditFieldLabel = uilabel(app.GridLayout7);
            app.MinstatnumEditFieldLabel.Layout.Row = 14;
            app.MinstatnumEditFieldLabel.Layout.Column = [4 5];
            app.MinstatnumEditFieldLabel.Text = 'Min stat num';

            % Create MinstatnumEditField
            app.MinstatnumEditField = uieditfield(app.GridLayout7, 'numeric');
            app.MinstatnumEditField.RoundFractionalValues = 'on';
            app.MinstatnumEditField.Editable = 'off';
            app.MinstatnumEditField.Layout.Row = 14;
            app.MinstatnumEditField.Layout.Column = 6;

            % Create RpxlEditFieldLabel
            app.RpxlEditFieldLabel = uilabel(app.GridLayout7);
            app.RpxlEditFieldLabel.Layout.Row = 15;
            app.RpxlEditFieldLabel.Layout.Column = 4;
            app.RpxlEditFieldLabel.Text = 'R(pxl)';

            % Create RpxlEditField
            app.RpxlEditField = uieditfield(app.GridLayout7, 'numeric');
            app.RpxlEditField.ValueDisplayFormat = '%11.6g';
            app.RpxlEditField.Editable = 'off';
            app.RpxlEditField.Layout.Row = 15;
            app.RpxlEditField.Layout.Column = [5 6];

            % Create TdegEditFieldLabel
            app.TdegEditFieldLabel = uilabel(app.GridLayout7);
            app.TdegEditFieldLabel.Layout.Row = 15;
            app.TdegEditFieldLabel.Layout.Column = 7;
            app.TdegEditFieldLabel.Text = 'T(deg)';

            % Create TdegEditField
            app.TdegEditField = uieditfield(app.GridLayout7, 'numeric');
            app.TdegEditField.ValueDisplayFormat = '%11.6g';
            app.TdegEditField.Editable = 'off';
            app.TdegEditField.Layout.Row = 15;
            app.TdegEditField.Layout.Column = [8 9];

            % Create avgsurfButton
            app.avgsurfButton = uibutton(app.GridLayout7, 'state');
            app.avgsurfButton.Text = 'avg surf';
            app.avgsurfButton.Layout.Row = 5;
            app.avgsurfButton.Layout.Column = [4 5];

            % Create surffilterButton
            app.surffilterButton = uibutton(app.GridLayout7, 'state');
            app.surffilterButton.Visible = 'off';
            app.surffilterButton.Text = 'surf filter';
            app.surffilterButton.Layout.Row = 5;
            app.surffilterButton.Layout.Column = [8 9];

            % Create meanEditFieldLabel
            app.meanEditFieldLabel = uilabel(app.GridLayout7);
            app.meanEditFieldLabel.HorizontalAlignment = 'right';
            app.meanEditFieldLabel.Visible = 'off';
            app.meanEditFieldLabel.Layout.Row = 6;
            app.meanEditFieldLabel.Layout.Column = 8;
            app.meanEditFieldLabel.Text = 'mean';

            % Create meanEditField
            app.meanEditField = uieditfield(app.GridLayout7, 'numeric');
            app.meanEditField.RoundFractionalValues = 'on';
            app.meanEditField.Visible = 'off';
            app.meanEditField.Layout.Row = 6;
            app.meanEditField.Layout.Column = 9;
            app.meanEditField.Value = -20;

            % Create rangEditFieldLabel
            app.rangEditFieldLabel = uilabel(app.GridLayout7);
            app.rangEditFieldLabel.HorizontalAlignment = 'right';
            app.rangEditFieldLabel.Visible = 'off';
            app.rangEditFieldLabel.Layout.Row = 7;
            app.rangEditFieldLabel.Layout.Column = 8;
            app.rangEditFieldLabel.Text = 'rang';

            % Create rangEditField
            app.rangEditField = uieditfield(app.GridLayout7, 'numeric');
            app.rangEditField.RoundFractionalValues = 'on';
            app.rangEditField.Visible = 'off';
            app.rangEditField.Layout.Row = 7;
            app.rangEditField.Layout.Column = 9;
            app.rangEditField.Value = 10;

            % Create findpkEditFieldLabel
            app.findpkEditFieldLabel = uilabel(app.GridLayout7);
            app.findpkEditFieldLabel.HorizontalAlignment = 'right';
            app.findpkEditFieldLabel.Visible = 'off';
            app.findpkEditFieldLabel.Layout.Row = 8;
            app.findpkEditFieldLabel.Layout.Column = 8;
            app.findpkEditFieldLabel.Text = 'find pk';

            % Create findpkEditField
            app.findpkEditField = uieditfield(app.GridLayout7, 'numeric');
            app.findpkEditField.RoundFractionalValues = 'on';
            app.findpkEditField.Visible = 'off';
            app.findpkEditField.Layout.Row = 8;
            app.findpkEditField.Layout.Column = 9;
            app.findpkEditField.Value = 20;

            % Create saveSurfButton
            app.saveSurfButton = uibutton(app.GridLayout7, 'push');
            app.saveSurfButton.Layout.Row = 4;
            app.saveSurfButton.Layout.Column = [8 9];
            app.saveSurfButton.Text = 'save Surf';

            % Create GridLayout3
            app.GridLayout3 = uigridlayout(app.GridLayout2);
            app.GridLayout3.ColumnWidth = {'1x'};
            app.GridLayout3.RowHeight = {'3x', '1x'};
            app.GridLayout3.Layout.Row = 1;
            app.GridLayout3.Layout.Column = 1;

            % Create GridLayout4
            app.GridLayout4 = uigridlayout(app.GridLayout3);
            app.GridLayout4.ColumnWidth = {'1x', '1x', '1x', '1x', '1x', '1x'};
            app.GridLayout4.RowHeight = {'1x', '1x', '1x', '1x', '1x', '1x', '1x', '1x', '1x', '1x'};
            app.GridLayout4.Layout.Row = 1;
            app.GridLayout4.Layout.Column = 1;

            % Create AddFilamentButton
            app.AddFilamentButton = uibutton(app.GridLayout4, 'push');
            app.AddFilamentButton.Layout.Row = 2;
            app.AddFilamentButton.Layout.Column = [1 3];
            app.AddFilamentButton.Text = 'Add Filament';

            % Create AcceptButton
            app.AcceptButton = uibutton(app.GridLayout4, 'push');
            app.AcceptButton.Layout.Row = 3;
            app.AcceptButton.Layout.Column = 1;
            app.AcceptButton.Text = 'Accept';

            % Create UndoButton
            app.UndoButton = uibutton(app.GridLayout4, 'push');
            app.UndoButton.Layout.Row = 3;
            app.UndoButton.Layout.Column = 2;
            app.UndoButton.Text = 'Undo';

            % Create CloseButton
            app.CloseButton = uibutton(app.GridLayout4, 'push');
            app.CloseButton.Layout.Row = 3;
            app.CloseButton.Layout.Column = 3;
            app.CloseButton.Text = 'Close';

            % Create ShowButton
            app.ShowButton = uibutton(app.GridLayout4, 'push');
            app.ShowButton.Layout.Row = 10;
            app.ShowButton.Layout.Column = 2;
            app.ShowButton.Text = 'Show';

            % Create SaveButton
            app.SaveButton = uibutton(app.GridLayout4, 'push');
            app.SaveButton.Layout.Row = 10;
            app.SaveButton.Layout.Column = 3;
            app.SaveButton.Text = 'Save';

            % Create FilamentsListBoxLabel
            app.FilamentsListBoxLabel = uilabel(app.GridLayout4);
            app.FilamentsListBoxLabel.Layout.Row = 4;
            app.FilamentsListBoxLabel.Layout.Column = [1 3];
            app.FilamentsListBoxLabel.Text = 'Filaments';

            % Create FilamentsListBox
            app.FilamentsListBox = uilistbox(app.GridLayout4);
            app.FilamentsListBox.Items = {};
            app.FilamentsListBox.Layout.Row = [5 9];
            app.FilamentsListBox.Layout.Column = [1 3];
            app.FilamentsListBox.Value = {};

            % Create TuneFilamentButton
            app.TuneFilamentButton = uibutton(app.GridLayout4, 'push');
            app.TuneFilamentButton.Layout.Row = 1;
            app.TuneFilamentButton.Layout.Column = [4 6];
            app.TuneFilamentButton.Text = 'Tune Filament';

            % Create NextptButton
            app.NextptButton = uibutton(app.GridLayout4, 'push');
            app.NextptButton.Layout.Row = 2;
            app.NextptButton.Layout.Column = 4;
            app.NextptButton.Text = 'Next pt';

            % Create PrevptButton
            app.PrevptButton = uibutton(app.GridLayout4, 'push');
            app.PrevptButton.Layout.Row = 2;
            app.PrevptButton.Layout.Column = 5;
            app.PrevptButton.Text = 'Prev pt';

            % Create AddptButton
            app.AddptButton = uibutton(app.GridLayout4, 'push');
            app.AddptButton.Layout.Row = 3;
            app.AddptButton.Layout.Column = 4;
            app.AddptButton.Text = 'Add pt';

            % Create MovptButton
            app.MovptButton = uibutton(app.GridLayout4, 'push');
            app.MovptButton.Layout.Row = 3;
            app.MovptButton.Layout.Column = 5;
            app.MovptButton.Text = 'Mov pt';

            % Create RmvptButton
            app.RmvptButton = uibutton(app.GridLayout4, 'push');
            app.RmvptButton.Layout.Row = 3;
            app.RmvptButton.Layout.Column = 6;
            app.RmvptButton.Text = 'Rmv pt';

            % Create SetVaButton
            app.SetVaButton = uibutton(app.GridLayout4, 'push');
            app.SetVaButton.Layout.Row = 8;
            app.SetVaButton.Layout.Column = 4;
            app.SetVaButton.Text = 'Set Va';

            % Create SetVbButton
            app.SetVbButton = uibutton(app.GridLayout4, 'push');
            app.SetVbButton.Layout.Row = 8;
            app.SetVbButton.Layout.Column = 5;
            app.SetVbButton.Text = 'Set Vb';

            % Create OptVsButton
            app.OptVsButton = uibutton(app.GridLayout4, 'push');
            app.OptVsButton.Layout.Row = 8;
            app.OptVsButton.Layout.Column = 6;
            app.OptVsButton.Text = 'Opt Vs';

            % Create UpdateButton
            app.UpdateButton = uibutton(app.GridLayout4, 'push');
            app.UpdateButton.Layout.Row = 10;
            app.UpdateButton.Layout.Column = 4;
            app.UpdateButton.Text = 'Update';

            % Create CancelButton
            app.CancelButton = uibutton(app.GridLayout4, 'push');
            app.CancelButton.Layout.Row = 9;
            app.CancelButton.Layout.Column = 5;
            app.CancelButton.Text = 'Cancel';

            % Create SFSTAButton
            app.SFSTAButton = uibutton(app.GridLayout4, 'push');
            app.SFSTAButton.Layout.Row = 9;
            app.SFSTAButton.Layout.Column = 4;
            app.SFSTAButton.Text = 'SF-STA';

            % Create FinishTuningButton
            app.FinishTuningButton = uibutton(app.GridLayout4, 'push');
            app.FinishTuningButton.Layout.Row = 10;
            app.FinishTuningButton.Layout.Column = [5 6];
            app.FinishTuningButton.Text = 'Finish Tuning';

            % Create CenteringButton
            app.CenteringButton = uibutton(app.GridLayout4, 'push');
            app.CenteringButton.Layout.Row = 5;
            app.CenteringButton.Layout.Column = [5 6];
            app.CenteringButton.Text = 'Centering';

            % Create TraceButton
            app.TraceButton = uibutton(app.GridLayout4, 'push');
            app.TraceButton.Layout.Row = 6;
            app.TraceButton.Layout.Column = 6;
            app.TraceButton.Text = 'Trace';

            % Create DeleteButton
            app.DeleteButton = uibutton(app.GridLayout4, 'push');
            app.DeleteButton.Layout.Row = 10;
            app.DeleteButton.Layout.Column = 1;
            app.DeleteButton.Text = 'Delete';

            % Create AbortButton
            app.AbortButton = uibutton(app.GridLayout4, 'push');
            app.AbortButton.Layout.Row = 9;
            app.AbortButton.Layout.Column = 6;
            app.AbortButton.Text = 'Abort';

            % Create thresEditFieldLabel
            app.thresEditFieldLabel = uilabel(app.GridLayout4);
            app.thresEditFieldLabel.HorizontalAlignment = 'right';
            app.thresEditFieldLabel.Layout.Row = 6;
            app.thresEditFieldLabel.Layout.Column = 4;
            app.thresEditFieldLabel.Text = 'thres';

            % Create thresEditField
            app.thresEditField = uieditfield(app.GridLayout4, 'numeric');
            app.thresEditField.Layout.Row = 6;
            app.thresEditField.Layout.Column = 5;

            % Create TomoButton
            app.TomoButton = uibutton(app.GridLayout4, 'push');
            app.TomoButton.Layout.Row = 1;
            app.TomoButton.Layout.Column = 2;
            app.TomoButton.Text = 'Tomo';

            % Create FilaButton
            app.FilaButton = uibutton(app.GridLayout4, 'push');
            app.FilaButton.Layout.Row = 1;
            app.FilaButton.Layout.Column = 3;
            app.FilaButton.Text = 'Fila';

            % Create LoadLabel
            app.LoadLabel = uilabel(app.GridLayout4);
            app.LoadLabel.Layout.Row = 1;
            app.LoadLabel.Layout.Column = 1;
            app.LoadLabel.Text = 'Load:';

            % Create GridLayout5
            app.GridLayout5 = uigridlayout(app.GridLayout3);
            app.GridLayout5.ColumnWidth = {'1x'};
            app.GridLayout5.RowHeight = {'1x', '5x'};
            app.GridLayout5.Layout.Row = 2;
            app.GridLayout5.Layout.Column = 1;

            % Create NotesTextAreaLabel
            app.NotesTextAreaLabel = uilabel(app.GridLayout5);
            app.NotesTextAreaLabel.Layout.Row = 1;
            app.NotesTextAreaLabel.Layout.Column = 1;
            app.NotesTextAreaLabel.Text = 'Notes';

            % Create NotesTextArea
            app.NotesTextArea = uitextarea(app.GridLayout5);
            app.NotesTextArea.Layout.Row = 2;
            app.NotesTextArea.Layout.Column = 1;

            % Create ContextMenu
            app.ContextMenu = uicontextmenu(app.UIFigure);

            % Create AutoEstimateMenu
            app.AutoEstimateMenu = uimenu(app.ContextMenu);
            app.AutoEstimateMenu.Text = 'Auto Estimate';
            
            % Assign app.ContextMenu
            app.thresEditField.ContextMenu = app.ContextMenu;

            % Show the figure after all components are created
            app.UIFigure.Visible = 'on';
        end
    end

    % App creation and deletion
    methods (Access = public)

        % Construct app
        function app = helis_exported

            % Create UIFigure and components
            createComponents(app)

            % Register the app with App Designer
            registerApp(app, app.UIFigure)

            % Execute the startup function
            runStartupFcn(app, @startupFcn)

            if nargout == 0
                clear app
            end
        end

        % Code that executes before app deletion
        function delete(app)

            % Delete UIFigure when app is deleted
            delete(app.UIFigure)
        end
    end
end