function subvols = get_subvols(src, poses, sz, varargin)
% poses should be a 3-by-N matrix represents the coordinates of pathes(at pixel).

persistent p;
if isempty(p)
    p = inputParser();
    p.addParameter('pad_by_rand', false);
    p.addParameter('pad_std', []);
    p.addParameter('pad_mean', []);
    p.addParameter('ignore_sub_pixel', false);
end
p.parse(varargin{:});

if isempty(p.Results.pad_std)
    pad_sigma = std(src(:));
else
    pad_sigma = p.Results.pad_std;
end

if isempty(p.Results.pad_std)
    pad_mu = mean(src, 'all');
else
    pad_mu = p.Results.pad_mean;
end

Npos = size(poses, 2);
sz = arrayexpand(sz, [1,3]);
if p.Results.pad_by_rand
    subvols = randn([sz+2, Npos]) * pad_sigma + pad_mu;
else
    subvols = zeros([sz+2, Npos]);
end

intpos = round(poses);
floatpos = poses - intpos;

pcen = cenpos(sz+2);

start_x_in_s = max(intpos(1,:)-pcen(1)+1, 1);
end_x_in_s = min(intpos(1,:)-pcen(1)+sz(1)+2, size(src,1));
start_y_in_s = max(intpos(2,:)-pcen(2)+1, 1);
end_y_in_s = min(intpos(2,:)-pcen(2)+sz(2)+2, size(src,2));
start_z_in_s = max(intpos(3,:)-pcen(3)+1, 1);
end_z_in_s = min(intpos(3,:)-pcen(3)+sz(3)+2, size(src,3));

for n = 1:Npos
    subvols(pcen(1)-intpos(1,n)+start_x_in_s(n):pcen(1)-intpos(1,n)+end_x_in_s(n),...
            pcen(2)-intpos(2,n)+start_y_in_s(n):pcen(2)-intpos(2,n)+end_y_in_s(n),...
            pcen(3)-intpos(3,n)+start_z_in_s(n):pcen(3)-intpos(3,n)+end_z_in_s(n),n) =...
            src(start_x_in_s(n):end_x_in_s(n), start_y_in_s(n):end_y_in_s(n), start_z_in_s(n):end_z_in_s(n));
end

if ~p.Results.ignore_sub_pixel
    subvols = voltranslation(subvols, -floatpos);
end

subvols = subvols(2:end-1,2:end-1,2:end-1,:);












end